@@ -129,36 +129,6 @@ if os.name == 'nt' and PY2:
         pass
 
 
-#==============================================================================
-# Cython support
-#==============================================================================
-RUN_CYTHON = os.environ.get("SPY_RUN_CYTHON") == "True"
-HAS_CYTHON = False
-
-if RUN_CYTHON:
-    try:
-        __import__('Cython')
-        HAS_CYTHON = True
-    except Exception:
-        pass
-
-    if HAS_CYTHON:
-        # Import pyximport to enable Cython files support for
-        # import statement
-        import pyximport
-        pyx_setup_args = {}
-
-        # Add Numpy include dir to pyximport/distutils
-        try:
-            import numpy
-            pyx_setup_args['include_dirs'] = numpy.get_include()
-        except Exception:
-            pass
-
-        # Setup pyximport and enable Cython files reload
-        pyximport.install(setup_args=pyx_setup_args, reload_support=True)
-
-
 #==============================================================================
 # Prevent subprocess.Popen calls to create visible console windows on Windows.
 # See issue #4932
@@ -534,8 +504,18 @@ class UserModuleReloader(object):
     def __init__(self, namelist=None, pathlist=None):
         if namelist is None:
             namelist = []
-        spy_modules = ['sitecustomize', 'spyder', 'spyderplugins']
+        else:
+            try:
+                namelist = namelist.split(',')
+            except Exception:
+                namelist = []
+
+        # Spyder modules
+        spy_modules = ['spyder_kernels']
+
+        # Matplotlib modules
         mpl_modules = ['matplotlib', 'tkinter', 'Tkinter']
+
         # Add other, necessary modules to the UMR blacklist
         # astropy: see issue 6962
         # pytorch: see issue 7041
@@ -550,12 +530,31 @@ class UserModuleReloader(object):
 
         if pathlist is None:
             pathlist = []
-        self.pathlist = pathlist
+        self.pathlist = self.create_pathlist(pathlist)
+
+        # List of previously loaded modules
         self.previous_modules = list(sys.modules.keys())
 
-    @property
-    def skip_paths(self):
-        """Python library paths to be skipped from module reloading."""
+        # List of module names to reload
+        self.modnames_to_reload = []
+
+        # Activate Cython support
+        self.has_cython = False
+        self.activate_cython()
+
+        # Check if the UMR is enabled or not
+        enabled = os.environ.get("SPY_UMR_ENABLED", "")
+        self.enabled = enabled.lower() == "true"
+
+        # Check if the UMR should print the list of reloaded modules or not
+        verbose = os.environ.get("SPY_UMR_VERBOSE", "")
+        self.verbose = verbose.lower() == "true"
+
+    def create_pathlist(self, initial_pathlist):
+        """
+        Add to pathlist Python library paths to be skipped from module
+        reloading.
+        """
         try:
             paths = sysconfig.get_paths()
             lib_paths = [paths['stdlib'],
@@ -563,22 +562,28 @@ class UserModuleReloader(object):
                          paths['scripts'],
                          paths['data']]
 
-            return lib_paths
+            return initial_pathlist + lib_paths
         except Exception:
-            return []
+            return initial_pathlist
 
-    def is_module_blacklisted(self, modname, modpath):
-        if HAS_CYTHON:
+    def is_module_reloadable(self, module, modname):
+        """Decide if a module is reloadable or not."""
+        if self.has_cython:
             # Don't return cached inline compiled .PYX files
-            return True
-        for path in [sys.prefix]+self.pathlist:
-            if modpath.startswith(path):
-                return True
+            return False
         else:
-            return set(modname.split('.')) & set(self.namelist)
+            if (self.is_module_in_pathlist(module) or
+                    self.is_module_in_namelist(modname)):
+                return False
+            else:
+                return True
 
-    def is_module_reloadable(self, module):
-        """Decide if a module can be reloaded or not."""
+    def is_module_in_namelist(self, modname):
+        """Decide if a module can be reloaded or not according to its name."""
+        return set(modname.split('.')) & set(self.namelist)
+
+    def is_module_in_pathlist(self, module):
+        """Decide if a module can be reloaded or not according to its path."""
         modpath = getattr(module, '__file__', None)
 
         # Skip module according to different criteria
@@ -586,12 +591,12 @@ class UserModuleReloader(object):
             # *module* is a C module that is statically linked into the
             # interpreter. There is no way to know its path, so we
             # choose to ignore it.
-            return False
-        elif any([p in modpath for p in self.skip_paths]):
+            return True
+        elif any([p in modpath for p in self.pathlist]):
             # We don't want to reload modules that belong to the
             # standard library or installed to site-packages,
             # just modules created by the user.
-            return False
+            return True
         elif not os.name == 'nt':
             # Module paths containing the strings below can be ihherited
             # from the default Linux installation or Homebrew in a
@@ -601,39 +606,74 @@ class UserModuleReloader(object):
                         r'^/usr/.*/dist-packages/.*',
                         r'^/Library/.*'
             ]
-            if [p for p in patterns if re.search(p, modpath)]:
-                return False
-            else:
-                return True
-        else:
-            return True
 
-    def run(self, verbose=False):
+            if [p for p in patterns if re.search(p, modpath)]:
+                return True
+            else:
+                return False
+        else:
+            return False
+
+    def activate_cython(self):
         """
-        Del user modules to force Python to deeply reload them
+        Activate Cython support.
+
+        We need to run this here because if the support is
+        active, we don't to run the UMR at all.
+        """
+        run_cython = os.environ.get("SPY_RUN_CYTHON") == "True"
+
+        if run_cython:
+            try:
+                __import__('Cython')
+                self.has_cython = True
+            except Exception:
+                pass
+
+            if self.has_cython:
+                # Import pyximport to enable Cython files support for
+                # import statement
+                import pyximport
+                pyx_setup_args = {}
+
+                # Add Numpy include dir to pyximport/distutils
+                try:
+                    import numpy
+                    pyx_setup_args['include_dirs'] = numpy.get_include()
+                except Exception:
+                    pass
+
+                # Setup pyximport and enable Cython files reload
+                pyximport.install(setup_args=pyx_setup_args,
+                                  reload_support=True)
+
+    def run(self):
+        """
+        Delete user modules to force Python to deeply reload them
 
         Do not del modules which are considered as system modules, i.e.
         modules installed in subdirectories of Python interpreter's binary
         Do not del C modules
         """
-        log = []
+        self.modnames_to_reload = []
         for modname, module in list(sys.modules.items()):
             if modname not in self.previous_modules:
                 # Decide if a module can be reloaded or not
-                if not self.is_module_reloadable(module):
+                if self.is_module_reloadable(module, modname):
+                    self.modnames_to_reload.append(modname)
+                    del sys.modules[modname]
+                else:
                     continue
 
-                # Reload module
-                if not self.is_module_blacklisted(modname, modpath):
-                    log.append(modname)
-                    del sys.modules[modname]
-
         # Report reloaded modules
-        if verbose and log:
+        if self.verbose and self.modnames_to_reload:
+            modnames = self.modnames_to_reload
             _print("\x1b[4;33m%s\x1b[24m%s\x1b[0m"\
-                   % ("Reloaded modules", ": "+", ".join(log)))
+                   % ("Reloaded modules", ": "+", ".join(modnames)))
 
-__umr__ = None
+
+__umr__ = UserModuleReloader(namelist=os.environ.get("SPY_UMR_NAMELIST",
+                                                     None))
 
 
 #==============================================================================
@@ -702,20 +742,6 @@ def _get_globals():
     return ipython_shell.user_ns
 
 
-def run_umr():
-    """Run the user module reloader."""
-    global __umr__
-    if os.environ.get("SPY_UMR_ENABLED", "").lower() == "true":
-        if __umr__ is None:
-            namelist = os.environ.get("SPY_UMR_NAMELIST", None)
-            if namelist is not None:
-                namelist = namelist.split(',')
-            __umr__ = UserModuleReloader(namelist=namelist)
-        else:
-            verbose = os.environ.get("SPY_UMR_VERBOSE", "").lower() == "true"
-            __umr__.run(verbose=verbose)
-
-
 def runfile(filename, args=None, wdir=None, namespace=None, post_mortem=False):
     """
     Run filename
@@ -729,7 +755,8 @@ def runfile(filename, args=None, wdir=None, namespace=None, post_mortem=False):
         # UnicodeError, TypeError --> eventually raised in Python 2
         # AttributeError --> systematically raised in Python 3
         pass
-    run_umr()
+    if __umr__.enabled:
+        __umr__.run()
     if args is not None and not isinstance(args, basestring):
         raise TypeError("expected a character buffer object")
     if namespace is None:
@@ -749,7 +776,7 @@ def runfile(filename, args=None, wdir=None, namespace=None, post_mortem=False):
         os.chdir(wdir)
     if post_mortem:
         set_post_mortem()
-    if HAS_CYTHON:
+    if __umr__.has_cython:
         # Cython files
         with io.open(filename, encoding='utf-8') as f:
             ipython_shell = get_ipython()
